# -*- coding: utf-8 -*-

# Copyright 2015 Donne Martin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

formatted_org = u'To view the avatar in your terminal, install the Python Image Library.\n\x1b[0m\x1b[0muser2\n\x1b[0m\x1b[32mbigcorp\n\x1b[0m\x1b[32minterwebz\n\x1b[0m\x1b[32muser2@foo.com\n\x1b[0m\x1b[36mOrganization\n\n\x1b[0mNo results found\x1b[0m'
formatted_user = u'To view the avatar in your terminal, install the Python Image Library.\n\x1b[0m\x1b[0muser1\n\x1b[0m\x1b[32mbigcorp\n\x1b[0m\x1b[32minterwebz\n\x1b[0m\x1b[32muser1@foo.com\n\x1b[0m\x1b[36mFollowers: 0 | \x1b[0m\x1b[36mFollowing: 0\n\n\x1b[0m\x1b[35m  1.   \x1b[0mrepo1 \x1b[0m()\x1b[0m\n\x1b[32m        Stars: 1      \x1b[0m\x1b[36mForks: 1      \x1b[0m\x1b[33mUpdated: just now \x1b[0m\n  View the page for \x1b[0m\x1b[35m1 through \x1b[0m\x1b[35m1\x1b[0m with the following command:\n\x1b[0m\x1b[35m    gh view [#] \x1b[0moptional: [-b/--browser] [--help]\n\x1b[0m\x1b[0m'
formatted_users = u'\x1b[35m  1.   \x1b[0mfoo1 \x1b[0m\n\x1b[35m  2.   \x1b[0mfoo2 \x1b[0m\n\x1b[35m  3.   \x1b[0mfoo3 \x1b[0m\n  View the page for \x1b[0m\x1b[35m1 through \x1b[0m\x1b[35m3\x1b[0m with the following command:\n\x1b[0m\x1b[35m    gh view [#] \x1b[0moptional: [-b/--browser] [--help]\n\x1b[0m\x1b[0m'
