# -*- coding: utf-8 -*-

# Copyright 2015 Donne Martin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

formatted_licenses = u'\x1b[35m  1.   \x1b[0mmit \x1b[0m\x1b[32m((MIT License))\x1b[0m\n\x1b[35m  2.   \x1b[0mgpl-2.0 \x1b[0m\x1b[32m((GNU General Public License v2.0))\x1b[0m\n\x1b[35m  3.   \x1b[0mbsd-2-clause \x1b[0m\x1b[32m((BSD 2-clause "Simplified" License))\x1b[0m\n\x1b[35m  4.   \x1b[0misc \x1b[0m\x1b[32m((ISC License))\x1b[0m\n\x1b[35m  5.   \x1b[0mepl-1.0 \x1b[0m\x1b[32m((Eclipse Public License 1.0))\x1b[0m\n\x1b[0m'
formatted_licenses_tip = u'  Run the following command to view or download a LICENSE file:\n    gh license apache-2.0\n    gh license apache-2.0 > LICENSE\n'
