# -*- coding: utf-8 -*-

# Copyright 2015 Donne Martin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

formatted_user_feed = u'\x1b[35m  1.   \x1b[0mLeonardoGitHub \x1b[0m\x1b[32mforked \x1b[0m\x1b[36mdonnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  2.   \x1b[0mnavroopsingh \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/dev-setup \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  3.   \x1b[0mprabaharanr \x1b[0m\x1b[32mforked \x1b[0m\x1b[36mdonnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  4.   \x1b[0mwdavis12 \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  5.   \x1b[0mvonec \x1b[0m\x1b[32mcommented on issue\x1b[0m\x1b[36m awslabs/aws-shell#115 \x1b[0m\x1b[33m(just now)\x1b[0m\n         \x1b[36m\x1b[0mFixed     \x1b[0m\n\x1b[35m  6.   \x1b[0mbenc \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/dev-setup \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  7.   \x1b[0mryusun \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  8.   \x1b[0mchirag200666 \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  9.   \x1b[0mravi-kothari \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/interactive-coding-challenges \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  10.  \x1b[0mravi-kothari \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  11.  \x1b[0mmarnovo \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  12.  \x1b[0mandersy005 \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/interactive-coding-challenges \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  13.  \x1b[0mandersy005 \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/viz \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  14.  \x1b[0mt9md \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/interactive-coding-challenges \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  15.  \x1b[0msilivizzle \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/dev-setup \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  16.  \x1b[0mahna \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  17.  \x1b[0mziyasal \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/saws \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  18.  \x1b[0mchefablz \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  19.  \x1b[0mkevinkn \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/dev-setup \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  20.  \x1b[0mfrrp \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/interactive-coding-challenges \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  21.  \x1b[0mfrrp \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  22.  \x1b[0mLiberTang0 \x1b[0m\x1b[32mforked \x1b[0m\x1b[36mdonnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  23.  \x1b[0mPhilippe-Caparroy \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  24.  \x1b[0mPhilippe-Caparroy \x1b[0m\x1b[32mforked \x1b[0m\x1b[36mdonnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  25.  \x1b[0mtiendan \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  26.  \x1b[0mSkyfold \x1b[0m\x1b[32mclosed issue\x1b[0m\x1b[36m awslabs/aws-shell#115 \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  27.  \x1b[0mSkyfold \x1b[0m\x1b[32mcommented on issue\x1b[0m\x1b[36m awslabs/aws-shell#115 \x1b[0m\x1b[33m(just now)\x1b[0m\n         \x1b[36m\x1b[0mFixed     \x1b[0m\n\x1b[35m  28.  \x1b[0msouadah \x1b[0m\x1b[32mforked \x1b[0m\x1b[36mdonnemartin/data-science-ipython-notebooks \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[35m  29.  \x1b[0mjoshuadsilva \x1b[0m\x1b[32mcommented on issue\x1b[0m\x1b[36m awslabs/aws-shell#115 \x1b[0m\x1b[33m(just now)\x1b[0m\n         \x1b[36m\x1b[0mWorks for me too img class="emoji" title=":+1:"\n         alt=":+1:" src="https://assets-\n         cdn.github.com/images/icons/emoji/unicode/1f44d.png" height="20"\n         width="20" align="absmiddle"\x1b[0m\n\x1b[35m  30.  \x1b[0mwilsantos99 \x1b[0m\x1b[32mstarred\x1b[0m\x1b[36m donnemartin/awesome-aws \x1b[0m\x1b[33m(just now)\x1b[0m\n\x1b[0m'
