# -*- coding: utf-8 -*-

# Copyright 2015 Donne Martin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

formatted_gitignores = u'\x1b[35m  1.   \x1b[0mActionscript \x1b[0m\n\x1b[35m  2.   \x1b[0mAda \x1b[0m\n\x1b[35m  3.   \x1b[0mAgda \x1b[0m\n\x1b[35m  4.   \x1b[0mAndroid \x1b[0m\n\x1b[35m  5.   \x1b[0mAppEngine \x1b[0m\n\x1b[0m'
formatted_gitignores_tip = u'  Run the following command to view or download a .gitignore file:\n    View:     gh gitignore Python\n    Download: gh gitignore Python > .gitignore\n'
