# -*- coding: utf-8 -*-

# Copyright 2015 Donne Martin. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

formatted_issues = u'\x1b[35m  1.   \x1b[0mtitle1 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/1)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n\x1b[35m  2.   \x1b[0mtitle2 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/2)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n\x1b[35m  3.   \x1b[0mtitle3 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/3)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n  View the page for \x1b[0m\x1b[35m1 through \x1b[0m\x1b[35m3\x1b[0m with the following command:\n\x1b[0m\x1b[35m    gh view [#] \x1b[0moptional: [-b/--browser] [--help]\n\x1b[0m\x1b[0m'
formatted_pull_requests = u'\x1b[35m  1.   \x1b[0mtitle1 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/1)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n\x1b[35m  2.   \x1b[0mtitle1 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/1)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n\x1b[35m  3.   \x1b[0mtitle1 \x1b[0m@user2 \x1b[0m\x1b[35m(user1/repo1/issues/1)\x1b[0m\n\x1b[32m        State: open       \x1b[0m\x1b[36mComments: 1     \x1b[0m\x1b[33mAssignee: user1      \x1b[0m\n  View the page for \x1b[0m\x1b[35m1 through \x1b[0m\x1b[35m3\x1b[0m with the following command:\n\x1b[0m\x1b[35m    gh view [#] \x1b[0moptional: [-b/--browser] [--help]\n\x1b[0m\x1b[0m'
