"""The module that holds much of the metadata about github3.py."""
__package_name__ = 'github3.py'
__title__ = 'github3'
__author__ = 'Ian Cordasco'
__author_email__ = 'graffatcolmingov@gmail.com'
__license__ = 'Modified BSD'
__copyright__ = 'Copyright 2012-2016 Ian Cordasco'
__version__ = '1.0.0a4'
__version_info__ = tuple(int(i) for i in __version__.split('.') if i.isdigit())
__url__ = 'http://github3.readthedocs.org'

__all__ = (
    '__package_name__', '__title__', '__author__', '__author_email__',
    '__license__', '__copyright__', '__version__', '__version_info__',
    '__url__',
)
